@@ -7,6 +7,7 @@ import copy
 import os
 import re
 import sys
+import inspect
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -31,7 +32,7 @@ except ImportError:
     pep8 = None
 
 
-__version__ = '0.7.1'
+__version__ = '0.7.2'
 
 
 PEP8_BIN = 'pep8'
@@ -84,6 +85,7 @@ class FixPEP8(object):
 
     [fixed method list]
         - e111
+        - e121,e122,e123,e124,e125,e126,e127,e128
         - e201,e202,e203
         - e211
         - e221,e222,e223,e224,e225
@@ -132,25 +134,24 @@ class FixPEP8(object):
 
     def _fix_source(self, results):
         completed_lines = []
-        global_read_fixes = ['e12']
         for result in sorted(results, key=_priority_key):
-            # Some fixes read all lines and thus will break if other fixes
-            # modify the lines.
-            is_global_read_fix = False
-            for prefix in global_read_fixes:
-                if result['id'].lower().startswith(prefix):
-                    is_global_read_fix = True
-
-            # Do not run global fix if any lines have been modified.
-            if is_global_read_fix and completed_lines:
-                continue
-
             if result['line'] in completed_lines:
                 continue
+
             fixed_methodname = "fix_%s" % result['id'].lower()
             if hasattr(self, fixed_methodname):
                 fix = getattr(self, fixed_methodname)
-                modified_lines = fix(result)
+
+                # Do not run logical fix if any lines have been modified.
+                is_logical_fix = len(inspect.getargspec(fix).args) > 2
+                if is_logical_fix and completed_lines:
+                    continue
+
+                if is_logical_fix:
+                    modified_lines = fix(result, self._get_logical(result))
+                else:
+                    modified_lines = fix(result)
+
                 if modified_lines:
                     completed_lines += modified_lines
                 elif modified_lines == []:  # Empty list means no fix
@@ -203,7 +204,7 @@ class FixPEP8(object):
 
     def find_logical(self, force=False):
         # make a variable which is the index of all the starts of lines
-        if force is False and self.logical_start is not None:
+        if not force and self.logical_start is not None:
             return
         logical_start = []
         logical_end = []
@@ -232,9 +233,12 @@ class FixPEP8(object):
         self.logical_start = logical_start
         self.logical_end = logical_end
 
-    def get_logical(self, result):
-        """return the logical line corresponding to the result.  Assumes input
-        is already E702-clean"""
+    def _get_logical(self, result):
+        """Return the logical line corresponding to the result.
+
+        Assumes input is already E702-clean.
+
+        """
         self.find_logical()
         row = result['line'] - 1
         col = result['column'] - 1
@@ -251,10 +255,12 @@ class FixPEP8(object):
         original = self.source[ls[0]:le[0] + 1]
         return ls, le, original
 
-    def _fix_reindent(self, result, fix_distinct=False):
-        """Fix a badly indented line by adding or removing from its initial
-        indent only."""
-        logical = self.get_logical(result)
+    def _fix_reindent(self, result, logical, fix_distinct=False):
+        """Fix a badly indented line.
+
+        This is done by adding or removing from its initial indent only.
+
+        """
         if not logical:
             return []
         ls, _, original = logical
@@ -294,51 +300,52 @@ class FixPEP8(object):
             self.source[line] = (
                 " " * (indent_to) + orig_line.lstrip()
             )
+            return [line + 1]  # Line indexed at 1
         else:
             return []
 
-    def fix_e121(self, result):
-        """The 'peculiar indent' error for hanging indents"""
+    def fix_e121(self, result, logical):
+        """The 'peculiar indent' error for hanging indents."""
         # fix by adjusting initial indent level
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e122(self, result):
-        """The 'absent indent' error for hanging indents"""
+    def fix_e122(self, result, logical):
+        """The 'absent indent' error for hanging indents."""
         # fix by adding an initial indent
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e123(self, result):
+    def fix_e123(self, result, logical):
         """The 'loose fingernails' indentation level error for hanging
-        indents"""
+        indents."""
         # fix by deleting whitespace to the correct level
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e124(self, result):
+    def fix_e124(self, result, logical):
         """The 'loose fingernails' indentation level error for visual
-        indents"""
+        indents."""
         # fix by inserting whitespace before the closing bracket
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e125(self, result):
+    def fix_e125(self, result, logical):
         """The 'often not visually distinct' error."""
         # fix by indenting the line in error to the next stop.
-        return self._fix_reindent(result, fix_distinct=True)
+        return self._fix_reindent(result, logical, fix_distinct=True)
 
-    def fix_e126(self, result):
-        """The 'spectacular indent' error for hanging indents"""
+    def fix_e126(self, result, logical):
+        """The 'spectacular indent' error for hanging indents."""
         # fix by deleting whitespace to the left
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e127(self, result):
+    def fix_e127(self, result, logical):
         """The 'interpretive dance' indentation error."""
         # fix by deleting whitespace to the correct level
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
-    def fix_e128(self, result):
+    def fix_e128(self, result, logical):
         """The 'I just wrap long lines with a line break in the middle of my
         argument list' indentation error."""
         # fix by deleting whitespace to the correct level
-        return self._fix_reindent(result)
+        return self._fix_reindent(result, logical)
 
     def fix_e201(self, result):
         line_index = result['line'] - 1
@@ -533,7 +540,7 @@ class FixPEP8(object):
                         target[c:].lstrip())
         self.source[result['line'] - 1] = fixed_source
 
-    def fix_e702(self, result):
+    def fix_e702(self, result, logical):
         """Fix multiple statements on one line."""
         line_index = result['line'] - 1
         target = self.source[line_index]
@@ -557,7 +564,9 @@ class FixPEP8(object):
         first = target[:offset].rstrip(';')
         second = target[offset:].lstrip(';')
 
-        f = [_get_indentation(target) + t.strip()
+        if not logical:
+            return []
+        f = [_get_indentation(logical[2][0]) + t.strip()
              for t in [first, second] if t.strip()]
 
         self.source[line_index] = self.newline.join(f) + self.newline
@@ -566,9 +575,6 @@ class FixPEP8(object):
         fixed_line = self.source[result['line'] - 1].rstrip()
         self.source[result['line'] - 1] = "%s%s" % (fixed_line, self.newline)
 
-    def fix_w292(self, _):
-        self.source[-1] += self.newline
-
     def fix_w293(self, result):
         assert not self.source[result['line'] - 1].strip()
         self.source[result['line'] - 1] = self.newline
@@ -1181,6 +1187,8 @@ class Wrapper(object):
         self.rel_indent = None
         sio = StringIO("".join(physical_lines))
         for t in tokenize.generate_tokens(sio.readline):
+            if not len(self.tokens) and t[0] in self.SKIP_TOKENS:
+                continue
             if t[0] != tokenize.ENDMARKER:
                 #if t[2][0] > max_seen:
                     #max_seen = t[2][0]
@@ -1191,10 +1199,11 @@ class Wrapper(object):
         )
 
     def build_tokens_logical(self, tokens):
-        """
-        Build a logical line from a list of tokens.  Returns the logical line
-        and a list of (offset, token) tuples.  Does not mute strings like the
-        version in pep8.py.
+        """Build a logical line from a list of tokens.
+
+        Returns the logical line and a list of (offset, token) tuples.  Does
+        not mute strings like the version in pep8.py.
+
         """
         # from pep8.py with minor modifications
         mapping = []
@@ -1234,6 +1243,7 @@ class Wrapper(object):
         Returns a list of lists; each list represents valid indent levels for
         the line in question, relative from the initial indent.  However, the
         first entry is the indent level which was expected.
+
         """
 
         # what follows is an adjusted version of
@@ -1299,7 +1309,7 @@ class Wrapper(object):
                         vi.append(indent[depth])
                     else:
                         # visual indent
-                        vi.append(rel_indent[open_row])
+                        vi.append(indent_level + rel_indent[open_row])
                 elif depth and indent[depth]:
                     # visual indent was previously confirmed.
                     vi.append(indent[depth])
@@ -1395,7 +1405,7 @@ class Wrapper(object):
                         if ind >= prev_indent:
                             del indent_chances[ind]
                     depth -= 1
-                    if depth:
+                    if depth and indent[depth]:  # modified
                         indent_chances[indent[depth]] = True
                     for idx in range(row, -1, -1):
                         if parens[idx]:
@@ -1499,7 +1509,7 @@ def parse_args(args):
                      'used')
 
     if opts.recursive and not (opts.in_place or opts.diff):
-        parser.error('must be used with --in-place or --diff')
+        parser.error('--recursive must be used with --in-place or --diff')
 
     if opts.in_place and opts.diff:
         parser.error('--in-place and --diff are mutually exclusive')
