@@ -254,14 +254,68 @@ def foo():
         line = """
 
 def foo_bar(baz, frop,
-    fizz, bang):
+    fizz, bang):  # E128
     pass
+
+if True:
+    x = {
+         }  # E123
+#: E121
+print "E121", (
+  "dent")
+#: E122
+print "E122", (
+"dent")
+#: E124
+print "E124", ("visual",
+               "indent_two"
+              )
+#: E125
+if (row < 0 or self.moduleCount <= row or
+    col < 0 or self.moduleCount <= col):
+    raise Exception("%s,%s - %s" % (row, col, self.moduleCount))
+#: E126
+print "E126", (
+            "dent")
+#: E127
+print "E127", ("over-",
+                  "over-indent")
+#: E128
+print "E128", ("under-",
+              "under-indent")
 """
         fixed = """
 
 def foo_bar(baz, frop,
-            fizz, bang):
+            fizz, bang):  # E128
     pass
+
+if True:
+    x = {
+    }  # E123
+#: E121
+print "E121", (
+    "dent")
+#: E122
+print "E122", (
+    "dent")
+#: E124
+print "E124", ("visual",
+               "indent_two"
+               )
+#: E125
+if (row < 0 or self.moduleCount <= row or
+        col < 0 or self.moduleCount <= col):
+    raise Exception("%s,%s - %s" % (row, col, self.moduleCount))
+#: E126
+print "E126", (
+    "dent")
+#: E127
+print "E127", ("over-",
+               "over-indent")
+#: E128
+print "E128", ("under-",
+               "under-indent")
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
@@ -283,6 +337,34 @@ sql = 'update %s set %s %s' % (from_table,
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e12_tricky(self):
+        line="""
+#: E126
+if (
+    x == (
+        3
+    ) or
+    x == (
+    3
+    ) or
+        y == 4):
+    pass
+"""
+        fixed = """
+#: E126
+if (
+    x == (
+        3
+    ) or
+    x == (
+        3
+    ) or
+        y == 4):
+    pass
+"""
+        self._inner_setup(line)
+        #self.assertEqual(self.result, fixed)
+
     def test_e191(self):
         line = """
 while True:
@@ -798,6 +880,27 @@ def foo():
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_indent_correctly(self):
+        line = """
+
+(
+    1,
+    2,
+    3); 4; 5; 5  # pyflakes
+"""
+        fixed = """
+
+(
+    1,
+    2,
+    3)
+4
+5
+5  # pyflakes
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e702_skip_with_triple_quote(self):
         # We do not support this yet.
         # We would expect '"""\n      hello\n   """\n1\n'.
@@ -827,10 +930,10 @@ class TestFixPEP8Warning(unittest.TestCase):
     def tearDown(self):
         os.remove(self.tempfile[1])
 
-    def _inner_setup(self, line):
+    def _inner_setup(self, line, options=()):
         with open(self.tempfile[1], 'w') as temp_file:
             temp_file.write(line)
-        opts, _ = autopep8.parse_args([self.tempfile[1]])
+        opts, _ = autopep8.parse_args([self.tempfile[1]] + list(options))
         sio = StringIO()
         autopep8.fix_file(filename=self.tempfile[1],
                           opts=opts,
@@ -852,7 +955,7 @@ class TestFixPEP8Warning(unittest.TestCase):
     def test_w292(self):
         line = "1\n2"
         fixed = "1\n2\n"
-        self._inner_setup(line)
+        self._inner_setup(line, options=['--select=W292'])
         self.assertEqual(self.result, fixed)
 
     def test_w293(self):
